import json
from pathlib import Path
from secrets import randbelow

from dotenv import dotenv_values
from typer.testing import CliRunner

from testing.util import get_random_string
from yak_server.cli import app

runner = CliRunner()


def test_yak_env_init() -> None:
    host = get_random_string(10)
    user_name = get_random_string(15)
    password = get_random_string(250)
    port = randbelow(80000)
    database = get_random_string(15)

    with runner.isolated_filesystem():
        result = runner.invoke(
            app,
            ["env", "init"],
            input=f"y\n{host}\n{user_name}\n{password}\n{port}\n{database}\n1800\n4\n",
        )

        assert result.exit_code == 0

        env = dotenv_values(Path(".env"))

        assert env["DEBUG"] == "1"
        assert env["JWT_EXPIRATION_TIME"] == "1800"
        assert env["JWT_SECRET_KEY"] is not None
        assert len(env["JWT_SECRET_KEY"]) == 256
        assert env["COMPETITION"] == "world_cup_2022"
        assert env["LOCK_DATETIME"] == "2022-11-20T17:00:00+01:00"
        assert env["OFFICIAL_RESULTS_URL"] == "https://en.wikipedia.org/wiki/2022_FIFA_World_Cup"
        assert env["DATA_FOLDER"] is not None
        assert Path(env["DATA_FOLDER"]).samefile(
            Path(__file__).parents[1] / "yak_server" / "data" / "world_cup_2022"
        )
        assert env["RULES"] is not None
        assert len(json.loads(env["RULES"])) == 2

        env_db = dotenv_values(Path(".env.db"))

        assert env_db["POSTGRES_HOST"] == host
        assert env_db["POSTGRES_USER_NAME"] == user_name
        assert env_db["POSTGRES_PASSWORD"] == password
        assert env_db["POSTGRES_PORT"] == str(port)
        assert env_db["POSTGRES_DB"] == database


def test_yak_env_init_production() -> None:
    with runner.isolated_filesystem():
        result = runner.invoke(
            app,
            ["env", "init"],
            input="n\ny\nroot\ny\n3000\ndb\n1800\n1\n",
        )

        assert result.exit_code == 0

        env = dotenv_values(Path(".env"))

        assert env["DEBUG"] == "0"


def test_yak_env_init_world_cup_2018() -> None:
    with runner.isolated_filesystem():
        result = runner.invoke(
            app,
            ["env", "init"],
            input="y\ny\nroot\nroot\ndddddddd\n\ndb\n1800\n3\n",
        )

        assert result.exit_code == 0

        env = dotenv_values(Path(".env"))

        assert env["RULES"] is not None
        assert json.loads(env["RULES"]) == {}



import re

import pytest

from scripts.compute_new_version import ReleaseType, compute_new_version


def test_compute_new_version() -> None:
    assert compute_new_version("1.30.2", ReleaseType.MAJOR) == "2.0.0"


def test_compute_new_minor_version() -> None:
    assert compute_new_version("0.5.3", ReleaseType.MINOR) == "0.6.0"


def test_compute_new_patch_version() -> None:
    assert compute_new_version("5.4.1", ReleaseType.PATCH) == "5.4.2"


def test_compute_incorrect_version() -> None:
    major, minor = 1, 3

    with pytest.raises(
        ValueError,
        match=re.escape(f"Incorrect version [{major}, {minor}]. Expecting 3 digits."),
    ):
        compute_new_version(f"{major}.{minor}", ReleaseType.MAJOR)


def test_compute_value_error() -> None:
    major, minor, patch = "a", 3, 6

    with pytest.raises(
        ValueError,
        match=re.escape(f"invalid literal for int() with base 10: '{major}'"),
    ):
        compute_new_version(f"{major}.{minor}.{patch}", ReleaseType.PATCH)



import argparse
from enum import Enum

NUMBER_OF_DIGITS = 3


class ReleaseType(Enum):
    MAJOR = "major"
    MINOR = "minor"
    PATCH = "patch"


def parse_version(version: str) -> list[int]:
    parsed_version = [int(version_digit) for version_digit in version.split(".")]

    if len(parsed_version) != NUMBER_OF_DIGITS:
        msg = f"Incorrect version {parsed_version}. Expecting 3 digits."
        raise ValueError(msg)

    return parsed_version


def unparse_version(major: int, minor: int, patch: int) -> str:
    return f"{major}.{minor}.{patch}"


def compute_new_version(current_version: str, release_type: ReleaseType) -> str:
    parsed_current_version = parse_version(current_version)

    if release_type == ReleaseType.MAJOR:
        return unparse_version(parsed_current_version[0] + 1, 0, 0)

    if release_type == ReleaseType.MINOR:
        return unparse_version(parsed_current_version[0], parsed_current_version[1] + 1, 0)

    return unparse_version(
        parsed_current_version[0],
        parsed_current_version[1],
        parsed_current_version[2] + 1,
    )


if __name__ == "__main__":  # pragma: no cover
    parser = argparse.ArgumentParser()
    parser.add_argument("current_version")
    parser.add_argument("release_type", choices=[e.value for e in ReleaseType])

    args = parser.parse_args()

    print(compute_new_version(args.current_version, ReleaseType(args.release_type)))

    raise SystemExit(0)
